from typing import Literal

from .. import OutputFormat

MimeType = Literal[
    "text/csv",
    "application/json",
    "text/plain",
    "text/xml",
    "application/vnd.ms-excel",
    "application/zip",
    "text/markdown",
    "text/html"
]


def get_mimetype(output_type: OutputFormat) -> MimeType:
    if output_type == "csv":
        return "text/csv"
    elif output_type == "json":
        return "application/json"
    elif output_type == "txt":
        return "text/plain"
    elif output_type == "xml":
        return "text/xml"
    elif output_type == "excel":
        return "application/vnd.ms-excel"
    elif output_type == "zip":
        return "application/zip"
    elif output_type == "markdown":
        return "text/markdown"
    elif output_type == "html":
        return "text/html"
    elif output_type == "txt":
        return "text/plain"
    else:
        raise ValueError(f"Unknown output type: {output_type}")



from enum import Enum
from typing import Literal, Set, Union, Optional

from pydantic import BaseModel, field_serializer


class FileToTransform(BaseModel):
    name: str
    file: str


class Error(BaseModel):
    message: str
    article: str
    transformer: str


class Pivot(BaseModel):
    identifiant: str
    journal: str
    journal_clean: str
    titre: str
    complement: str
    annee: int
    mois: int
    jour: int
    heure: int
    minute: int
    seconde: int
    date: str
    epoch: int
    auteur: str
    texte: str
    keywords: Set[str]
    langue: str
    url: str

    def __hash__(self):
        return hash((self.journal, self.date, self.titre))

    @field_serializer('keywords')
    def serialize_keywords(self, kw: Set[str]):
        return ', '.join(kw)


OutputFormat = Literal["csv", "excel", "json", "txt", "xml", "zip"]

class TXM_MODE(str, Enum):
    MULTIPLE_FILES = "multiple_files"
    ONE_FILE_PB = "one_file_pb"
    LEGACY = "legacy"

class Outputs(str, Enum):
    iramuteq = "iramuteq"
    txm = "txm"
    json = "json"
    csv = "csv"
    excel = "excel"
    markdown = "markdown"
    stats = "stats"
    processedStats = "processedStats"
    staticGraphs = "staticGraphs"
    dynamicGraphs = "dynamicGraphs"


class TransformerOutput(BaseModel):
    data: Union[str, bytes, None]
    output: OutputFormat
    filename: str


class Params:
    def __init__(
            self,
            filter_keywords: bool = False,
            filter_lang: bool = False,
            minimal_support: int = 1,
            minimal_support_kw: Optional[int] = None,
            minimal_support_journals: Optional[int] = None,
            minimal_support_authors: Optional[int] = None,
            minimal_support_dates: Optional[int] = None,
            txm_mode: Union[TXM_MODE, str] = TXM_MODE.MULTIPLE_FILES,
            keep_p_tags: bool = False,
    ):
        assert all(
            (isinstance(x, int) and x > 0) or x is None
             for x in [
                minimal_support,
                minimal_support_kw,
                minimal_support_journals,
                minimal_support_authors,
                minimal_support_dates
            ]
        )
        assert all(isinstance(x, bool) for x in [filter_keywords, filter_lang, keep_p_tags])

        self.filter_keywords: bool = filter_keywords
        self.filter_lang: bool = filter_lang
        self.minimal_support: int = minimal_support
        self.minimal_support_kw: int = minimal_support_kw or minimal_support
        self.minimal_support_journals: int = minimal_support_journals or minimal_support
        self.minimal_support_authors: int = minimal_support_authors or minimal_support
        self.minimal_support_dates: int = minimal_support_dates or minimal_support
        self.txm_mode: TXM_MODE = TXM_MODE(txm_mode) if isinstance(txm_mode, str) else txm_mode if isinstance(txm_mode, TXM_MODE) else TXM_MODE.MULTIPLE_FILES
        self.keep_p_tags: bool = keep_p_tags



import json
from typing import List

from .models import Pivot, FileToTransform


def adjust_fields(elem: dict) -> dict:
    elem["keywords"] = set(e.strip() for e in elem.get("keywords", "").split(","))

    return elem


def do_one_file(file: FileToTransform) -> List[Pivot]:
    return [
        Pivot.parse_obj(
            adjust_fields(elem)
        )
        for elem in json.loads(file.file).values()
    ]


def pivots_from_files(
        files: List[FileToTransform],
) -> List[Pivot]:
    pivots = []

    for path in files:
        try:
            pivots.extend(do_one_file(path))
        except Exception as e:
            raise ValueError(f"Error parsing pivot file {path.name}: {e}")

    return pivots



import hashlib
import io
import zipfile
from collections import defaultdict

from typing import List

import yaml

from ..models import Pivot, TransformerOutput
from ..transformers.transformer import Transformer
from ..utils import super_writestr


class MarkdownTransformer(Transformer):
    def __init__(self):
        super().__init__()
        self.output_type = "zip"
        self.output = TransformerOutput(
            data=None, output=self.output_type, filename=f'{self.name}_output.{self.output_type}'
        )
        self.seen_names = set()
        self.stats = None
        self.stats_output = ""

    def generate_markdown(self, pivot: Pivot):
        # Générer le contenu du fichier markdown
        frontmatter = {
            "journal": pivot.journal_clean,
            "auteur": pivot.auteur,
            "titre": pivot.titre,
            "date": pivot.date,
            "langue": self.clean_string(pivot.langue),
            "tags": [self.clean_string(tag) for tag in pivot.keywords],
            "journal_charts": "journal_" + self.clean_string(pivot.journal_clean),
            "auteur_charts": "auteur_" + self.clean_string(pivot.journal_clean),
            "url": pivot.url,
        }

        markdown_content = f"---\n{yaml.dump(frontmatter)}---\n\n{pivot.texte}"

        # Nom du fichier markdown
        # Si le titre est trop long, on le tronque à 100 caractères
        # Si le titre est vide (une fois nettoyé), on utilise le hash du texte
        base_nom = self.clean_string(pivot.titre).strip("_")[:100] or hashlib.md5(pivot.texte.encode()).hexdigest()

        nom = f"{frontmatter['journal']}/{base_nom}.md"
        if nom in self.seen_names:
            # Si le nom existe déjà, on ajoute la date à la fin (sans l'heure)
            nom = f"{nom[:-3]}_{self.clean_string(pivot.date).split('t')[0]}.md"
        self.seen_names.add(nom)

        return nom, markdown_content

    def transform(self, pivots: List[Pivot]) -> TransformerOutput:
        try:
            self.compute_stats(pivots)
            in_memory_zip = io.BytesIO()
            with zipfile.ZipFile(in_memory_zip, "w", zipfile.ZIP_DEFLATED) as zipf:
                for pivot in pivots:
                    filename, content = self.generate_markdown(pivot)
                    super_writestr(zipf, filename, content)
                zipf.writestr("Statistiques.md", self.make_waffle())

            in_memory_zip.seek(0)
            self.output.data = in_memory_zip.getvalue()
            return self.output
        except Exception as e:
            print(self.__class__.__name__, e)
            raise

    def compute_stats(self, pivots, key="journal"):
        articles_par_cle = defaultdict(int)

        # Parcourez la liste d'éléments pivots
        for pivot in pivots:
            # Incrémentez le compteur pour le journal actuel
            articles_par_cle[pivot.journal_clean] += 1

        # Convertissez le defaultdict en un dictionnaire Python standard pour la sortie
        self.stats = dict(articles_par_cle)

    def make_waffle(self):
        output = "## Articles par journal \n\n" \
                 "```chartsview\n"
        chart = {
            "type": "Treemap",
            "data": {
                "name": "root",
                "children": [],
            },
            "options": {
                "colorField": "name",
                "enableSearchInteraction": {
                    "field": "journal_chart"
                }
            }
        }
        for journal, value in self.stats.items():
            # do this to avoid searching in the Statistics.md file
            search_value = 'journal_' + self.clean_string(journal) + '" -file:(Statistiques) "'
            chart['data']['children'].append({'name': journal, 'value': value, 'journal_chart': search_value})

        output += yaml.dump(chart)
        output += "```"
        return output




import logging
import os
import zipfile
from io import StringIO, BytesIO
from pathlib import Path
from typing import Annotated  # , Optional
from uuid import uuid4
from zipfile import ZipFile
from datetime import date

from fastapi import FastAPI, UploadFile, Request, HTTPException, File, Form  # , Query
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from starlette.responses import FileResponse, StreamingResponse
from filedetect import FileDetect

from europarser import FileToTransform, pipeline
from europarser.api.utils import get_mimetype
from europarser.models import TransformerOutput, Params, Outputs

# root_dir = os.path.dirname(__file__)
root_dir = Path(__file__).parent
host = os.getenv('EUROPARSER_SERVER', '')
temp_dir = Path(os.getenv('EUROPARSER_TEMP_DIR', '/tmp/europarser'))
temp_dir.mkdir(parents=True, exist_ok=True)
app = FastAPI()

app.mount("/static", StaticFiles(directory=root_dir / "static"), name="static")
templates = Jinja2Templates(directory=root_dir / "templates")
favicon_path = root_dir / "static/favicon.ico"

logger = logging.getLogger("europarser_api.api")
logger.setLevel(logging.DEBUG)


@app.get("/", response_class=HTMLResponse)
async def read_root(request: Request):
    return templates.TemplateResponse('index.html', {'request': request, 'host': host})


@app.get("/favicon.ico", include_in_schema=False)
async def favicon():
    return FileResponse(favicon_path)


@app.get("/create_file_upload_url")
async def create_file_upload_url():
    uuid_ = uuid4().hex
    tmp_folder = temp_dir / uuid_
    if tmp_folder.exists():
        raise HTTPException(status_code=500, detail="UUID collision")
    tmp_folder.mkdir()
    print(f"Created folder {tmp_folder}")
    return {"uuid": uuid_, "upload_url": f"/upload/{uuid_}"}


@app.post("/upload/{uuid}")
async def upload_file(
        uuid: str,
        file: Annotated[UploadFile, File(...)],
):
    tmp_folder = temp_dir / uuid
    if not tmp_folder.exists():
        raise HTTPException(status_code=404, detail="UUID not found")
    with open(tmp_folder / file.filename, "wb") as f:
        f.write(file.file.read())
    return {"file": file.filename}


@app.post("/convert")
async def convert(
        uuid: Annotated[str, Form(...)],
        output: Annotated[list[Outputs], Form(...)],
        # params: Annotated[dict, Form(...)],
        filter_keywords: Annotated[bool, Form(...)] = None,
        filter_lang: Annotated[bool, Form(...)] = None,
        minimal_support: Annotated[int, Form(...)] = None,
        minimal_support_kw: Annotated[int, Form(...)] = None,
        minimal_support_journals: Annotated[int, Form(...)] = None,
        minimal_support_authors: Annotated[int, Form(...)] = None,
        minimal_support_dates: Annotated[int, Form(...)] = None,
        txm_mode: Annotated[str, Form(...)] = None,
        keep_p_tags: Annotated[bool, Form(...)] = None,
):
    folder = temp_dir / uuid

    if not folder.exists():
        raise HTTPException(status_code=404, detail="UUID not found")

    to_process_files = FileDetect.find(path=folder, suffixes={".html", ".json"}).result
    all_files = FileDetect.find(path=folder).result

    if len(to_process_files) == 0:
        raise HTTPException(status_code=404, detail="No files found")
    elif len(to_process_files) != len(all_files):
        raise HTTPException(status_code=400, detail=f"Only HTML files are supported.\nList of files :{all_files}")

    # parse all files
    try:
        to_process = [FileToTransform(name=f.name, file=f.read_text(encoding="utf-8")) for f in to_process_files]
    except UnicodeDecodeError:
        raise HTTPException(status_code=400, detail="Invalid File Provided")

    try:
        params = Params(
            **{k: v for k, v in {
                "filter_keywords": filter_keywords,
                "filter_lang": filter_lang,
                "minimal_support": minimal_support,
                "minimal_support_kw": minimal_support_kw,
                "minimal_support_journals": minimal_support_journals,
                "minimal_support_authors": minimal_support_authors,
                "minimal_support_dates": minimal_support_dates,
                "txm_mode": txm_mode,
                "keep_p_tags": keep_p_tags,
            }.items() if v is not None}
        )
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))

    try:
        # process result
        results: list[TransformerOutput] = pipeline(to_process, output, params)
    except NotImplementedError as e:
        raise HTTPException(status_code=500, detail=f"A unimplemented output was requested\nError:{e}")
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))

    # if only one output was required let's return a single file
    if len(results) == 1:
        result = results[0]

        if isinstance(result.data, StringIO) or isinstance(result.data, BytesIO):
            pass
        elif not isinstance(result.data, bytes):
            result.data = StringIO(result.data)
        else:
            result.data = BytesIO(result.data)

        return StreamingResponse(
            result.data,
            media_type=get_mimetype(result.output),
            headers={
                'Content-Disposition': f"attachment; filename=EuroParser_{date.today().strftime('%d-%m-%Y')}_{result.filename}"}
        )

    # else let's create a zip with all files
    zip_io = BytesIO()
    with ZipFile(zip_io, mode='w', compression=zipfile.ZIP_DEFLATED) as temp_zip:
        for result in results:
            logger.info(f"Adding {result.filename} to zip")
            if result.output == "zip":
                name = Path(result.filename).stem  # get filename without extension (remove .zip basically)
                logger.info(f"Zip file detected, extracting {name}")
                with ZipFile(BytesIO(result.data), mode='r') as z:
                    for f in z.namelist():
                        temp_zip.writestr(f"{name}/{f}", z.read(f))
                continue

            temp_zip.writestr(f"{result.filename}", result.data)

    zip_io.seek(0)
    return StreamingResponse(
        zip_io,
        media_type="application/zip",
        headers={'Content-Disposition': f'attachment; filename=EuroParser_{date.today().strftime("%d-%m-%Y")}.zip'}
    )


def main():
    from argparse import ArgumentParser

    import uvicorn

    parser = ArgumentParser()
    parser.add_argument("--host", default="127.0.0.1", help="Host to bind to")
    parser.add_argument("--port", default=8000, help="Port to bind to", type=int)
    args = parser.parse_args()
    uvicorn.run(app, host=args.host, port=args.port)


if __name__ == "__main__":
    main()



import xml.dom.minidom as dom
import zipfile
from io import BytesIO, StringIO
from typing import List, Optional, Any
from xml.sax.saxutils import quoteattr, escape

from ..models import Pivot, TransformerOutput, TXM_MODE, Params
from ..transformers.transformer import Transformer
from ..utils import super_writestr


class TXMTransformer(Transformer):
    XML_HEADER = '<?xml version="1.0" encoding="UTF-8"?>\n'

    def __init__(self, params: Optional[Params] = None, **kwargs: Optional[Any]):
        super().__init__(params, **kwargs)
        self.output_type = "zip" if self.params.txm_mode == TXM_MODE.MULTIPLE_FILES else "xml"
        self.output = TransformerOutput(
            data=None, output=self.output_type, filename=f'{self.name}_output.{self.output_type}'
        )

    def clean_name(self, pivot: Pivot, i=None) -> str:
        return f"{pivot.annee}_{pivot.mois}_{pivot.jour}_{pivot.journal_clean.replace(' ', '-')}_{self.clean_string(pivot.titre)[:50]}{f'_{i}' if i else ''}.xml"

    def do_one_article(self, pivot: Pivot, stream, pb=False) -> None:
        pivot_dict = {
            "identifiant": pivot.identifiant,
            "titre": pivot.titre,
            "date": pivot.date,
            "journal": pivot.journal,
            "auteur": pivot.auteur,
            "annee": pivot.annee,
            "mois": pivot.mois,
            "jour": pivot.jour,
            "journalClean": pivot.journal_clean,
            "keywords": ', '.join(pivot.keywords),
            "langue": pivot.langue,
            "url": pivot.url,
        }
        stream.write("<text ")
        for key, value in pivot_dict.items():
            if value:
                stream.write(f"{key}={quoteattr(str(value))} ")

        cleaned_string = escape(pivot.texte.strip().replace('\n', '<lb/>\n')) if pivot.texte else ''
        stream.write(f"> {cleaned_string} </text>\n")
        if pb:
            stream.write("<pb/>\n")

    def transform(self, pivot_list: List[Pivot], mode=None) -> TransformerOutput:
        try:
            if mode:
                self.params.txm_mode = mode
                self.output_type = "zip" if self.params.txm_mode == TXM_MODE.MULTIPLE_FILES else "xml"
                self.output = TransformerOutput(
                    data=None, output=self.output_type, filename=f'{self.name}_output.{self.output_type}'
                )

            if self.params.txm_mode in {TXM_MODE.LEGACY, TXM_MODE.ONE_FILE_PB}:
                with StringIO() as f:
                    f.write(self.XML_HEADER + "<corpus>\n")
                    length = len(pivot_list)
                    for i, pivot in enumerate(pivot_list):
                        if i + 1 == length:
                            self.do_one_article(pivot, f)
                        else:
                            self.do_one_article(pivot, f, pb=self.params.txm_mode == TXM_MODE.ONE_FILE_PB)
                    f.write("</corpus>")
                    self.output.data = dom.parseString(f.getvalue()).toprettyxml()
            elif self.params.txm_mode == TXM_MODE.MULTIPLE_FILES:
                with BytesIO() as zio:
                    with zipfile.ZipFile(zio, 'w', compression=zipfile.ZIP_DEFLATED) as z:
                        for pivot in pivot_list:


                            with StringIO() as f:
                                f.write(self.XML_HEADER)
                                self.do_one_article(pivot, f)
                                try:
                                    super_writestr(
                                        z,
                                        self.clean_name(pivot),
                                        dom.parseString(f.getvalue()).toprettyxml()
                                    )
                                except Exception as e:
                                    pass
                                    raise e

                    self.output.data = zio.getvalue()
            else:
                raise ValueError(f"Invalid TXM mode: {self.params.txm_mode}")

            return self.output
        except Exception as e:
            print(self.__class__.__name__, e)
            raise


from io import BytesIO
from typing import List

from ..models import Pivot, TransformerOutput
from ..transformers.transformer import Transformer

import polars as pl

class ExcelTransformer(Transformer):
    def __init__(self):
        super(ExcelTransformer, self).__init__()
        self.output_type = "excel"
        self.output = TransformerOutput(data=None, output=self.output_type,
                                        filename=f'{self.name}_output.xlsx')

    def transform(self, pivot_list: List[Pivot]) -> TransformerOutput:
        try:
            df = pl.DataFrame([p.model_dump() for p in pivot_list])
            with BytesIO() as output:
                df.write_excel(output)
                output.seek(0)
                self.output.data = output.read()
            return self.output
        except Exception as e:
            print(self.__class__.__name__, e)
            raise



import concurrent.futures
import json
import logging
import re
from io import StringIO
from datetime import datetime
from collections import Counter
from hashlib import sha256
from typing import Optional, Any, Union

from bs4 import BeautifulSoup, element, Tag
from tqdm.auto import tqdm

from .daniel_light import get_KW
from .lang_detect import detect_lang
from .models import FileToTransform, Pivot, Params
from .transformers.transformer import Transformer
from .utils import find_datetime


class BadArticle(Exception):
    pass


class PivotTransformer(Transformer):
    journal_split = re.compile(r"\(| -|,? no. | \d|  | ;|\.fr")
    double_spaces_and_beyond = re.compile(r"(\s{2,})")

    def __init__(self, params: Optional[Params] = None, **kwargs: Optional[Any]) -> None:
        super().__init__(params, **kwargs)
        self._logger.setLevel(logging.DEBUG)
        self.corpus = []
        self.bad_articles = []
        self.ids = set()
        self.all_keywords = Counter()
        self.doublons_count = 0
        self.articles_count = 0
        self.good_articles_count = 0

    def clean_name(self, doc):
        return f"{doc.year}_{doc.month}_{doc.day}_{doc.journal}_{self.clean_string(doc.titre)[:50]}"

    def subspaces(self, s: str) -> str:
        return self.double_spaces_and_beyond.sub(r"\1", s).strip()

    def to_text_with_p(self, doc_text: Tag) -> str:
        """
        Utility function to convert a bs4 Tag to text, keeping the <p> tags
        :param doc_text: bs4 Tag
        :return: str
        """
        if not isinstance(doc_text, Tag):
            raise ValueError("doc_text is not a bs4 Tag")
        
        if not self.params.keep_p_tags:
            return self.subspaces(doc_text.text.strip())

        content = doc_text.contents
        if not content:
            return ""

        sio = StringIO()
        for e in content:
            if isinstance(e, str):
                sio.write(e.strip())
            elif isinstance(e, Tag):
                if e.name == "p":
                    sio.write(f"<p>{e.text.strip()}</p>")
                else:
                    sio.write(e.text.strip())
            else:
                raise ValueError(f"Unknown type {type(e)} in doc_text")

        return self.subspaces(sio.getvalue())



    def transform(self, files_to_transform: list[FileToTransform]) -> list[Pivot]:
        for file in files_to_transform:
            self._logger.debug("Processing file " + file.name)
            soup = BeautifulSoup(file.file, 'lxml')
            articles = soup.find_all("article")

            self.articles_count += len(articles)

            with concurrent.futures.ThreadPoolExecutor() as executor:
                futures = [executor.submit(self.transform_article, article) for article in articles]
                concurrent.futures.wait(futures, return_when=concurrent.futures.ALL_COMPLETED)

        self._logger.info(f"Nombre d'articles : {len(self.corpus)}")

        self.persist_json()
        self._persist_errors(datetime.now().strftime("%Y%m%d"))
        self.apply_parameters()

        if self.doublons_count:
            self._logger.warn(f"Nombre d'articles doublons : {self.doublons_count}")
        else:
            self._logger.info("Pas d'articles doublons")

        self._logger.info(f"Nombre d'articles traités : {self.good_articles_count}")
        self._logger.info(f"Nombre d'articles au total : {self.articles_count}")

        return sorted(self.corpus, key=lambda x: x.epoch)

    def transform_article(
            self,
            article: Union[BeautifulSoup, element.Tag],
    ) -> None:
        assert isinstance(article, (BeautifulSoup, element.Tag)), "article is not a BeautifulSoup object"
        try:
            doc = {
                "identifiant": None,
                "journal": None,
                "date": None,
                "annee": None,
                "mois": None,
                "jour": None,
                "heure": None,
                "minute": None,
                "seconde": None,
                "epoch": None,
                "titre": None,
                "complement": None,
                "texte": None,
                "auteur": "Unknown",
                "journal_clean": None,
                "keywords": None,
                "langue": "UNK",
                "url": None,
            }
            try:
                doc["journal"] = self.subspaces(article.find("span", attrs={"class": "DocPublicationName"}).text)
            except Exception as e:
                self._logger.debug("pas un article de presse")
                raise BadArticle("journal")

            try:
                doc_header = article.find("span", attrs={"class": "DocHeader"}).text
            except AttributeError:
                doc_header = ""

            try:
                doc_sub_section = article.find(
                    "span",
                    attrs={"class": "DocTitreSousSection"}
                ).find_next_sibling("span").text
            except AttributeError:
                doc_sub_section = ""

            try:
                datetime_ = find_datetime(doc_header or doc_sub_section)
            except ValueError:
                raise BadArticle("datetime")

            if datetime_:
                doc.update({
                    "date": datetime_.strftime("%Y %m %dT%H:%M:%S"),
                    "annee": datetime_.year,
                    "mois": datetime_.month,
                    "jour": datetime_.day,
                    "heure": datetime_.hour,
                    "minute": datetime_.minute,
                    "seconde": datetime_.second,
                    "epoch": datetime_.timestamp()
                })

            try:
                doc_titre_full = article.find("div", attrs={"class": "titreArticle"})
                assert doc_titre_full is not None
            except AssertionError:
                try:
                    doc_titre_full = article.find("p", attrs={"class": "titreArticleVisu"})
                    assert doc_titre_full is not None
                except AssertionError:
                    raise BadArticle("titre")

            try:
                doc["titre"] = doc_titre_full.find("p", attrs={
                    "class": "sm-margin-TopNews titreArticleVisu rdp__articletitle"}).text
            except AttributeError:
                try:
                    doc["titre"] = doc_titre_full.find("div", attrs={"class": "titreArticleVisu"}).text
                except AttributeError:
                    try:
                        doc["titre"] = doc_titre_full.text
                    except AttributeError:
                        raise BadArticle("titre")

            doc["titre"] = self.subspaces(doc["titre"])

            try:
                doc_bottomNews = doc_titre_full.find("p", attrs={"class": "sm-margin-bottomNews"}).text
                if not doc_bottomNews:
                    raise AttributeError
            except AttributeError:
                doc_bottomNews = ""

            try:
                doc_subtitle = doc_titre_full.find("p", attrs={"class": "sm-margin-TopNews rdp__subtitle"}).text
                if not doc_subtitle:
                    raise AttributeError
            except AttributeError:
                doc_subtitle = ""

            doc["complement"] = self.subspaces(
                " | ".join((doc_header, doc_sub_section, doc_bottomNews, doc_subtitle))
            )

            try:
                doc_text = article.find("div", attrs={"class": "docOcurrContainer"})
                assert doc_text is not None and doc_text.text.strip()
            except AssertionError:
                if article.find("div", attrs={"class": "DocText clearfix"}) is None:
                    raise BadArticle("texte")
                else:
                    doc_text = article.find("div", attrs={"class": "DocText clearfix"})

            doc["url"] = ""
            for u in doc_text.select("a"):
                if "Cet article est paru dans" in u.get_text():
                    doc["url"] = u.get("href")
                    break

            doc["texte"] = self.to_text_with_p(doc_text)

            doc_auteur = doc_titre_full.find_next_sibling('p')

            if doc_auteur and "class" in doc_auteur.attrs and doc_auteur.attrs['class'] == ['sm-margin-bottomNews']:
                doc["auteur"] = self.subspaces(doc_auteur.text.strip().lower())

            # on garde uniquement le titre (sans les fioritures)
            journal_clean = self.journal_split.split(doc["journal"])[0]
            doc["journal_clean"] = self.subspaces(journal_clean)

            doc["keywords"] = get_KW(doc["titre"], doc["texte"])

            self.all_keywords.update(doc["keywords"])

            identifiant = sha256(
                ' '.join(
                    (doc["titre"], doc["journal"], doc["date"])
                ).encode()).hexdigest()

            langue = detect_lang(doc["texte"])
            if langue:
                doc["langue"] = langue

            if identifiant not in self.ids:
                doc["identifiant"] = identifiant
                self.corpus.append(Pivot(**doc))
                self.ids.add(identifiant)
            else:
                self._logger.warn(
                    "Article déjà présent dans le corpus : "
                    f"{doc['titre'] = }, {doc['date'] = }, {doc['journal'] = }, {identifiant = }"
                )
                self._add_error(ValueError("Article déjà présent dans le corpus"), article)
                self.doublons_count += 1

            self.good_articles_count += 1

        except BadArticle as e:
            if self._logger.isEnabledFor(logging.DEBUG):
                self._add_error(e, article)
                self.bad_articles.append(article)

        # To avoid the exception to be lost when catched by the ThreadPoolExecutor
        except Exception as e:
            print(e)
            raise e

        return

    def apply_parameters(self) -> list[Pivot]:
        if self.params.filter_keywords is True:
            for article in self.corpus:
                article.keywords = list(filter(lambda kw: self.filter_kw(kw, self.params.minimal_support_kw or 1), article.keywords))

        return self.corpus

    def filter_kw(self, keyword: str, minimum_shared_kw: int = 1) -> bool:
        return self.all_keywords[keyword] > minimum_shared_kw

    def get_bad_articles(self) -> None:
        print(self.bad_articles)

    def persist_json(self) -> None:
        """
        utility function to persist the result of the pivot transformation
        """
        if not self.output_path:
            return

        json_ver = json.dumps({i: article.dict() for i, article in enumerate(self.corpus)}, ensure_ascii=False)

        output_file = self.output_path / f"{sha256(json_ver.encode()).hexdigest()}.json"

        with output_file.open("w", encoding="utf-8") as f:
            f.write(json_ver)


if __name__ == "__main__":
    import cProfile
    import pstats

    from pathlib import Path

    pr = cProfile.Profile()
    pr.enable()

    p = PivotTransformer()

    for file in tqdm(list(Path("/home/marceau/Nextcloud/eurocollectes").glob("**/*.HTML"))):
        with file.open(mode="r", encoding="utf-8") as f:
            p.transform(FileToTransform(file=f.read(), name=file.name))

    p.get_bad_articles()

    pr.disable()
    ps = pstats.Stats(pr).sort_stats('cumulative')
    ps.print_stats()



from typing import List

from ..models import Pivot
from ..transformers.transformer import Transformer


class GephiTransformer(Transformer):
    def __init__(self):
        super(GephiTransformer, self).__init__()

    def transform(self, pivot_list: List[Pivot], graph_type="skeletton") -> str:
        try:
            keyword_to_authors = {}
            res = "Source;Target;edge_label"
            for pivot in pivot_list:
                for keyword in pivot.keywords.split(','):
                    if keyword.strip() == "":
                        continue
                    keyword_to_authors[keyword] = keyword_to_authors.get(keyword, set())
                    keyword_to_authors[keyword].add(pivot.auteur)
            # pour chaque mot clé
            for keyword, author_list in keyword_to_authors.items():
                # pour chaque auteur de la liste des auteurs
                for index in range(len(list(author_list))):
                    author = list(author_list)[index]
                    if author.lower() != "unknown":
                        # ce code là sert pour créer des liens entre auteurs, les résultats ne sont pas probants
                        # # pour chaque autre auteur de la liste des auteurs
                        # for sub_index, author in enumerate(list(author_list)[index:]):
                        #     if sub_index < len(list(author_list)[index:]) - 2:
                        #         # on créé un lien
                        #         res += f"\n{author};{list(author_list)[sub_index + 1]};{keyword}"
                        # celui ci pour créer des liens auteur -> mot clé
                        res += f"\n{author.lower()};{keyword};aucun"

            return res
            # code pour le graphe temporel si on le fait un jour
            # pivot_list.sort(key=lambda x: x.date)
            # first_date = pivot_list[0].date
            # last_date = pivot_list[-1].date

            # return df.to_csv(sep=",", index=False)
        except Exception as e:
            print(self.__class__.__name__, e)
            raise


from pathlib import Path
from typing import Optional

from filedetect import FileDetect

try:
    from . import pipeline
    from .models import FileToTransform, TransformerOutput, Params, Outputs, TXM_MODE
except ImportError:
    from europarser import pipeline
    from europarser.models import FileToTFileToTransform, TransformerOutput, Params, Outputs, TXM_MODE

def main(folder: Path | str, outputs: list[Outputs], params: Optional[Params] = None) -> None:
    if params is None:
        params = Params()

    if isinstance(folder, str):
        folder = Path(folder)
    elif not isinstance(folder, Path):
        raise ValueError(f"folder must be a Path or a string, not {type(folder)}")


    files = FileDetect.find(
        path=folder,
        suffixes={".html", ".json"},
    )
    if len(files) == 0:
        raise ValueError(f"No files found in {folder}")

    print(f"Found {len(files)} files")

    files = [
        FileToTransform(name=f.name, file=f.read_text(encoding="utf-8")) for f in files
    ]

    # process result
    results: list[TransformerOutput] = pipeline(files, outputs, params=params)

    # write results
    folder = folder / 'results'
    folder.mkdir(exist_ok=True)
    for result in results:
        print(result.filename)
        if isinstance(result.data, str):
            with open(folder / f'{result.filename}', 'w', encoding='utf-8') as f:
                f.write(result.data)
        else:
            with open(folder / f'{result.filename}', 'wb') as f:
                f.write(result.data)



if __name__ == '__main__':
    folder = Path('/mnt/P2/PycharmProjects/EuropressParser/test_emile')
    # outputs = ["json", "txm", "iramuteq", "csv", "excel", "stats", "processedStats", "markdown", "dynamicGraphs"]
    # outputs = ["json", "stats", "processed_stats", "plots"]
    outputs = ["txm"]
    params = Params(
        minimal_support_kw=5,
        minimal_support_authors=2,
        minimal_support_journals=8,
        minimal_support_dates=3,
        filter_keywords=True,
        filter_lang=True,
        txm_mode="multiple_files"
    )
    main(folder, outputs, params=params)
